==> atlassian_connect_jwt-0.3.4.tar.gz: lib/src/product_host_token.dart <==
/**
 * Supports creating and validating jwt tokens for interaction with Atlassian
 * product hosts
 */
library ac_dart.ac_jwt.product_host_token;

import 'utils.dart';
import 'qsh.dart';
export 'qsh.dart';

import 'package:dart_jwt/dart_jwt.dart';
import 'package:quiver/collection.dart';

import 'package:logging/logging.dart';

Logger _log = new Logger('ac_dart.ac_jwt.product_host_token');

/**
 * Creates a Jwt token suitable for inclusion in requests to Atlassian product
 * hosts
 */
String createProductHostToken(String path, String method, Map queryParams,
    String sharedSecret, String issuer,
    {String subject, String audience}) {
  final Multimap<String, String> mm = _toMultiMap(queryParams);
  final String qsh = createQshFromPath(path, method, mm);
  final DateTime now = new DateTime.now();
  // TODO: need to configure how long it is valid for
  final claimSet = new ProductHostClaimSet(
      issuer, subject, now.add(const Duration(minutes: 3)), now, qsh, audience);

  _log.finest('created claimSet: \n${claimSet.toJson()}');
  final jwt = new JsonWebToken.jws(
      claimSet, new JwaSymmetricKeySignatureContext(sharedSecret));
  return jwt.encode();
}

/**
 * Decodes a Jwt token received from an Atlassian product host
 */
JsonWebToken<ProductHostClaimSet> decodeProductHostToken(String jwtToken,
    {JwsValidationContext validationContext}) {
  return new JsonWebToken.decode(jwtToken,
      validationContext: validationContext,
      claimSetParser: (Map json) => new ProductHostClaimSet.fromJson(json));
}

/**
 * A [JwtClaimSet] for use with Atlassian product host communication. It 
 * contains one custom claim - [queryStringHash]
 */

class ProductHostClaimSet extends OpenIdJwtClaimSet {
  final String queryStringHash;

  ProductHostClaimSet(String issuer, String subject, DateTime expiry,
      DateTime issuedAt, this.queryStringHash, String audience)
      : super(issuer, subject, expiry, issuedAt, [audience]);

  ProductHostClaimSet.mut(
      {String issuer,
      String subject,
      DateTime expiry,
      DateTime issuedAt,
      String queryStringHash,
      String audience})
      : this(issuer, subject, expiry, issuedAt, queryStringHash, audience);

  ProductHostClaimSet.fromJson(Map json)
      : queryStringHash = json['qsh'],
        super.fromJson(json);

  @override
  Map toJson() {
    return removeNullValues(super.toJson()..['qsh'] = queryStringHash);
  }

  @override
  Set<ConstraintViolation> validate(
      ProductHostClaimSetValidationContext validationContext) {
    return super.validate(validationContext)
      ..addAll(_validateQsh(validationContext));
  }

  Set<ConstraintViolation> _validateQsh(
      ProductHostClaimSetValidationContext validationContext) {
    final String expectedQsh = validationContext.qshFactory();
    return queryStringHash == expectedQsh
        ? new Set.identity()
        : (new Set()
          ..add(new ConstraintViolation(
              "Query String Hash mismatch. Expected '$expectedQsh'. Got '$queryStringHash'")));
  }
}

typedef String QshFactory();

QshFactory defaultQshFactory(Uri nonCanonicalUri, String method,
    [String contextPath]) {
  return () => createQsh(nonCanonicalUri, method, contextPath);
}

class ProductHostClaimSetValidationContext
    extends JwtClaimSetValidationContext {
  final QshFactory qshFactory;

  ProductHostClaimSetValidationContext.withFactory(this.qshFactory,
      {Duration expiryTolerance: const Duration(seconds: 30)})
      : super(expiryTolerance: expiryTolerance);

  ProductHostClaimSetValidationContext(Uri nonCanonicalUri, String method,
      {String contextPath,
      Duration expiryTolerance: const Duration(seconds: 30)})
      : this.withFactory(
            defaultQshFactory(nonCanonicalUri, method, contextPath),
            expiryTolerance: expiryTolerance);
}

Multimap<String, String> _toMultiMap(Map m) {
  final mm = new Multimap();
  m.forEach((k, v) => mm.add(k, v));
  return mm;
}

==> atlassian_connect_jwt-0.3.5.tar.gz: lib/src/product_host_token.dart <==
/**
 * Supports creating and validating jwt tokens for interaction with Atlassian
 * product hosts
 */
library ac_dart.ac_jwt.product_host_token;

import 'utils.dart';
import 'qsh.dart';
export 'qsh.dart';

import 'package:dart_jwt/dart_jwt.dart';
import 'package:quiver/collection.dart';

import 'package:logging/logging.dart';

Logger _log = new Logger('ac_dart.ac_jwt.product_host_token');

/**
 * Creates a Jwt token suitable for inclusion in requests to Atlassian product
 * hosts
 */
String createProductHostToken(String path, String method, Map queryParams,
    String sharedSecret, String issuer,
    {String subject, String audience}) {
  final Multimap<String, String> mm = _toMultiMap(queryParams);
  final String qsh = createQshFromPath(path, method, mm);
  final DateTime now = new DateTime.now();
  // TODO: need to configure how long it is valid for
  final claimSet = new ProductHostClaimSet(
      issuer, subject, now.add(const Duration(minutes: 3)), now, qsh, audience);

  _log.finest('created claimSet: \n${claimSet.toJson()}');
  final jwt = new JsonWebToken.jws(
      claimSet, new JwaSymmetricKeySignatureContext(sharedSecret));
  return jwt.encode();
}

/**
 * Decodes a Jwt token received from an Atlassian product host
 */
JsonWebToken<ProductHostClaimSet> decodeProductHostToken(String jwtToken,
    {JwsValidationContext validationContext}) {
  return new JsonWebToken.decode(jwtToken,
      validationContext: validationContext,
      claimSetParser: (Map json) => new ProductHostClaimSet.fromJson(json));
}

/**
 * A [JwtClaimSet] for use with Atlassian product host communication. It 
 * contains one custom claim - [queryStringHash]
 */

class ProductHostClaimSet extends OpenIdJwtClaimSet {
  final String queryStringHash;

  ProductHostClaimSet(String issuer, String subject, DateTime expiry,
      DateTime issuedAt, this.queryStringHash, String audience)
      : super(issuer, subject, expiry, issuedAt, [audience]);

  ProductHostClaimSet.mut(
      {String issuer,
      String subject,
      DateTime expiry,
      DateTime issuedAt,
      String queryStringHash,
      String audience})
      : this(issuer, subject, expiry, issuedAt, queryStringHash, audience);

  ProductHostClaimSet.fromJson(Map json)
      : queryStringHash = json['qsh'],
        super.fromJson(_doctorExp(json));

  @override
  Map toJson() {
    return removeNullValues(super.toJson()..['qsh'] = queryStringHash);
  }

  @override
  Set<ConstraintViolation> validate(
      ProductHostClaimSetValidationContext validationContext) {
    return super.validate(validationContext)
      ..addAll(_validateQsh(validationContext));
  }

  Set<ConstraintViolation> _validateQsh(
      ProductHostClaimSetValidationContext validationContext) {
    final String expectedQsh = validationContext.qshFactory();
    return queryStringHash == expectedQsh
        ? new Set.identity()
        : (new Set()
          ..add(new ConstraintViolation(
              "Query String Hash mismatch. Expected '$expectedQsh'. Got '$queryStringHash'")));
  }
}

Map _doctorExp(Map json) {
  // WTF hipchat
  json['exp'] = (json['exp'] as num).toInt();
  return json;
}

typedef String QshFactory();

QshFactory defaultQshFactory(Uri nonCanonicalUri, String method,
    [String contextPath]) {
  return () => createQsh(nonCanonicalUri, method, contextPath);
}

class ProductHostClaimSetValidationContext
    extends JwtClaimSetValidationContext {
  final QshFactory qshFactory;

  ProductHostClaimSetValidationContext.withFactory(this.qshFactory,
      {Duration expiryTolerance: const Duration(seconds: 30)})
      : super(expiryTolerance: expiryTolerance);

  ProductHostClaimSetValidationContext(Uri nonCanonicalUri, String method,
      {String contextPath,
      Duration expiryTolerance: const Duration(seconds: 30)})
      : this.withFactory(
            defaultQshFactory(nonCanonicalUri, method, contextPath),
            expiryTolerance: expiryTolerance);
}

Multimap<String, String> _toMultiMap(Map m) {
  final mm = new Multimap();
  m.forEach((k, v) => mm.add(k, v));
  return mm;
}
